#ifndef AlienCommand_h
#define AlienCommand_h

// $Id: AlienCommand.h,v 1.1 2012/01/20 08:16:59 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"

#include <QString>
#include <QStringList>
#include <list>
#include <set>

//! handles top commands
class AlienCommand: public Counter
{

    public:

    //! enumeration for command type
    enum Type
    {

        NONE,
        JOB_LIST,
        USER_LIST,
        DEFAULT_USER,

        TRACE,
        TRACE_JDL,
        TRACE_ALL,

        JOB_FILES,

        RESUBMIT,
        RESUBMIT_MASTER,

        COPY,

        KILL,
        KILL_MASTER,

    };

    //! list of commands
    typedef std::list<AlienCommand> List;

    //! constructor
    AlienCommand( const Type = NONE );

    //! less than operator
    bool operator < ( const AlienCommand& other ) const
    {
        if( priority() != other.priority() ) return priority() < other.priority();
        else return id() < other.id();
    }

    //! equal to operator
    bool operator == (const AlienCommand& other ) const
    { return priority() == other.priority() && id() == other.id(); }

    //!@name accessors
    //@{

    //! command name
    QString name( void ) const
    { return name( type() ); }

    //! map Type to command name
    static QString name( const Type& type );

    //! id
    unsigned int id( void ) const
    { return id_; }

    //! priority
    unsigned int priority( void ) const
    { return priority_; }

    //! type
    Type type( void ) const
    { return type_; }

    // true if arguments are passed
    bool hasArguments( void ) const
    { return !arguments_.isEmpty(); }

    //! arguments
    const QStringList& arguments( void ) const
    { return arguments_; }

    //@}

    //!@name modifiers
    //@{

    //! priority
    void setPriority( unsigned int value )
    { priority_ = value; }

    //! add argument
    AlienCommand& addArgument( const QString& argument )
    {
        arguments_.push_back( argument );
        return *this;
    }

    //! push arguments
    AlienCommand& operator << (const QString& argument )
    { return addArgument( argument ); }

    //! clear arguments
    void clearArguments( void )
    { arguments_.clear(); }

    //@}

    //! use to sort command based on id and arguments
    class CommandSortFTor
    {

        public:

        bool operator() (const AlienCommand& first, const AlienCommand& second )
        {

            // check type
            if( first.type() != second.type() ) return first.type() < second.type();

            // check arguments
            for( int index = 0; index < std::min( first.arguments().size(), second.arguments().size() ); ++index )
            { if( first.arguments()[index] != second.arguments()[index] ) return first.arguments()[index] < second.arguments()[index]; }

            // check arguments size
            return first.arguments().size() < second.arguments().size();

        }

    };

    typedef std::set<AlienCommand, CommandSortFTor> Set;

    private:

    //! static id counter
    static unsigned int& _counter( void );

    //! map types to command name
    typedef std::map<Type,QString> NameMap;

    //! name map
    static NameMap& _names( void );

    //! unique id
    unsigned int id_;

    //! priority
    unsigned int priority_;

    //! type
    Type type_;

    //! arguments
    QStringList arguments_;

    //! initialize command names
    static void _initializeNames( void );

};

#endif
