// $Id: AlienCommandModel.cpp,v 1.1 2012/01/20 08:17:00 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    AlienCommandModel.h
\brief   Stores ftp commands for display in lists
\author  Hugo Pereira
\version $Revision: 1.1 $
\date    $Date: 2012/01/20 08:17:00 $
*/

#include "CustomPixmap.h"
#include "AlienCommandModel.h"
#include "Icons.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>
#include <cassert>

//_______________________________________________
const QString AlienCommandModel::columnTitles_[ AlienCommandModel::nColumns ] =
{ "Id", "Name", "Arguments" };

//_______________________________________________________________
AlienCommandModel::AlienCommandModel( QObject* parent ):
    ListModel<AlienCommand>( parent ),
    Counter( "AlienCommandModel" )
{

    Debug::Throw( "AlienCommandModel::AlienCommandModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant AlienCommandModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated command
    const AlienCommand& command( get()[index.row()] );

    // return text associated to command and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case ID:
            return command.id();

            case NAME:
            return command.name();

            case ARGUMENTS:
            return command.arguments().join( " " );

            default:
            return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == NAME )
    { return _icon(command.type()); }

    return QVariant();

}

//__________________________________________________________________
QVariant AlienCommandModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//____________________________________________________________
void AlienCommandModel::add( AlienCommand command )
{
    Debug::Throw() << "AlienCommandModel::add - " << command.name() << " id: " << command.id() << endl;
    emit layoutAboutToBeChanged();
    _get().push_back( command );
    emit layoutChanged();
}

//____________________________________________________________
void AlienCommandModel::remove( AlienCommand command )
{
    Debug::Throw() << "AlienCommandModel::remove - " << command.name() << " id: " << command.id() << endl;

    List::iterator iter = find( _get().begin(), _get().end(), command );
    if( iter != _get().end() )
    {
        emit layoutAboutToBeChanged();
        _get().erase( iter );
        emit layoutChanged();
    }

    return;

}

//_________________________________________________________
void AlienCommandModel::updateConfiguration( void )
{

    Debug::Throw( "AlienCommandModel::updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();

    reset();

}

//_____________________________________________________________________
void AlienCommandModel::_installIcons( void ) const
{

    Debug::Throw( "AlienCommandList::_installIcons" );

    if( !_icons().empty() ) return;

    // load pixmaps
    QSize size( IconSize( IconSize::Medium ) );
    QSize scale = 0.75*size;

    // type icons
    typedef std::map< AlienCommand::Type, QString > IconNames;
    IconNames type_names;
    type_names[AlienCommand::JOB_LIST] = ICONS::RELOAD;
    type_names[AlienCommand::USER_LIST] = ICONS::USERS;
    type_names[AlienCommand::DEFAULT_USER] = ICONS::USERS;
    type_names[AlienCommand::RESUBMIT] = ICONS::RESUBMIT;
    type_names[AlienCommand::KILL] = ICONS::KILL;
    type_names[AlienCommand::RESUBMIT_MASTER] = ICONS::RESUBMIT;
    type_names[AlienCommand::KILL_MASTER] = ICONS::KILL;
    type_names[AlienCommand::TRACE] = ICONS::INFORMATION;
    type_names[AlienCommand::TRACE_ALL] = ICONS::INFORMATION;
    type_names[AlienCommand::TRACE_JDL] = ICONS::INFORMATION;

    // load link pixmap
    for( IconNames::iterator iter = type_names.begin(); iter != type_names.end(); ++iter )
    {

        _icons()[iter->first] = CustomPixmap().empty( size )
            .merge( CustomPixmap().find( iter->second )
            .scaled( scale, Qt::KeepAspectRatio, Qt::SmoothTransformation ),
            CustomPixmap::CENTER );

    }

    return;
}

//_______________________________________________
AlienCommandModel::IconCache& AlienCommandModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
QIcon AlienCommandModel::_icon( AlienCommand::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter == _icons().end() )
    {
        QIcon icon( CustomPixmap().empty( IconSize( IconSize::Medium ) ) );
        _icons()[type] = icon;
        return icon;
    } else return iter->second;

}
