// $Id: Application.cpp,v 1.41 2012/01/20 17:48:30 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file Application.cc
\brief application Main Window singleton object
\author  Hugo Pereira
\version $Revision: 1.41 $
\date $Date: 2012/01/20 17:48:30 $
*/

#include "Application.h"
#include "ConfigurationDialog.h"
#include "IconEngine.h"
#include "Icons.h"
#include "AlienCommandDialog.h"
#include "JobInformationDialog.h"
#include "AlienInterface.h"
#include "JobSet.h"
#include "JobStatusBar.h"
#include "LogDialog.h"
#include "LogWidget.h"
#include "MainWindow.h"
#include "QtUtil.h"
#include "ScratchFileMonitor.h"
#include "TreeView.h"
#include "UserSet.h"
#include "Util.h"

#include <QtGui/QApplication>



//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings )
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( SERVER::ApplicationManager::commandLineParser() );
    out.registerOption( "--user", "string", "user name from whom jobs will be shown" );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void Application::usage( void )
{
    Debug::Throw(0) << "usage : Top [options]" << endl;
    commandLineParser().usage();
    return;
}

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments ),
    Counter( "Application" ),
    interface_( 0 ),
    mainWindow_(0),
    logDialog_(0)
{

    Debug::Throw( "Application::Application.\n" );
    connect( this, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );

}

//____________________________________________
Application::~Application( void )
{ if( interface_ ) delete interface_; }

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // interface
    interface_ = new AlienInterface();

    // scratch files
    scratchFileMonitor_ = new ScratchFileMonitor( this );
    connect( qApp, SIGNAL( aboutToQuit( void ) ), scratchFileMonitor_, SLOT( deleteScratchFiles( void ) ) );

    // actions
    _installActions();

    // main window
    mainWindow_ = new MainWindow();
    mainWindow().setWindowTitle( "gridwatch" );

    // log
    logDialog_ = new LogDialog(0);
    connect( logDialog_, SIGNAL( dialogHidden() ), SLOT( _logDialogHidden() ) );

    // connection between job manager and interface
    connect( &mainWindow(), SIGNAL( commandRequested( AlienCommand ) ), &interface(), SLOT( addCommand( AlienCommand ) ) );
    connect( &mainWindow(), SIGNAL( activeUsersChanged( UserSet ) ), &interface(), SLOT( updateActiveUsers( UserSet ) ) );
    connect( &mainWindow(), SIGNAL( jobInformationRequested( const Job& ) ), SLOT( _requestJobInformation( const Job& ) ) );
    connect( &interface(), SIGNAL( defaultUserAvailable( QString ) ), &mainWindow(), SLOT( updateDefaultUser( QString ) ) );
    connect( &interface(), SIGNAL( userListAvailable( UserSet ) ), &mainWindow(), SLOT( processUserList( UserSet ) ) );
    connect( &interface(), SIGNAL( jobListAvailable( QString, const JobSet& ) ), &mainWindow(), SLOT( processJobList( QString, const JobSet& ) ) );
    connect( &interface(), SIGNAL( jobInformationAvailable( const Job&, const QString& ) ), &mainWindow(), SLOT( processJobInformation( const Job& ) ) );
    connect( &interface(), SIGNAL( jobChildrenAvailable( const Job&, const Job::IdSet& ) ), &mainWindow(), SLOT( processJobChildren( const Job&, const Job::IdSet& ) ) );
    connect( &interface(), SIGNAL( messageAvailable( const QString& ) ), &mainWindow().statusBar().label(), SLOT( setTextAndUpdate( const QString& ) ) );

    connect( &interface(), SIGNAL( commandAdded( AlienCommand ) ), &_model(), SLOT( add( AlienCommand ) ) );
    connect( &interface(), SIGNAL( commandCompleted( AlienCommand ) ), &mainWindow(), SLOT( commandCompleted( AlienCommand ) ) );
    connect( &interface(), SIGNAL( commandCompleted( AlienCommand ) ), &_model(), SLOT( remove( AlienCommand ) ) );

    // top widget
    mainWindow().centerOnDesktop();
    mainWindow().show();
    mainWindow().installFrames();

    // configuration
    emit configurationChanged();

    mainWindow().requestDefaultUser();
    mainWindow().requestUserList();

    Debug::Throw( "Application::realizeWidget - done.\n" );
    return true;

}

//___________________________________________________________
void Application::closeJobInformationDialogs( void )
{

    Debug::Throw( "Application::closeJobInformationDialogs.\n" );

    // need local copy of dialog map because the latter might get modified while the dialogs are closed
    BASE::KeySet<JobInformationDialog> dialogs( this );
    for( BASE::KeySet<JobInformationDialog>::iterator iter = dialogs.begin(); iter != dialogs.end(); ++iter )
    { (*iter)->close(); }

}

//___________________________________________________________
void Application::_installActions( void )
{

    Debug::Throw( "Application::_installActions.\n" );

    // rename about action
    aboutAction().setText( "About gridwatch" );
    configurationAction().setText( "Configure gridwatch ..." );

    // command list
    commandListAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Command List", this );
    connect( commandListAction_, SIGNAL( triggered() ), SLOT( _showCommandList() ) );

    // log
    logAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Command Log", this );
    logAction_->setEnabled( false );
    connect( logAction_, SIGNAL( triggered() ), SLOT( _showLog() ) );

    toggleLogAction_ = new QAction( "Enable Command Log", this );
    toggleLogAction_->setCheckable( true );
    toggleLogAction_->setChecked( false );
    connect( toggleLogAction_, SIGNAL( toggled( bool ) ), SLOT( _toggleLog( bool ) ) );

    // hide killed jobs
    hideKilledJobsAction_ = new QAction( "Hide Killed Jobs", this );
    hideKilledJobsAction_->setCheckable( true );
    hideKilledJobsAction_->setChecked( false );
    connect( hideKilledJobsAction_, SIGNAL( toggled( bool ) ), SLOT( _toggleHideKilledJobs( bool ) ) );


    // pause
    pauseAction_ = new QAction( IconEngine::get( ICONS::PAUSE ), "Pause", this );
    pauseAction_->setCheckable( true );
    pauseAction_->setChecked( false );
    connect( pauseAction_, SIGNAL( toggled( bool ) ), &interface(), SLOT( togglePause( bool ) ) );

}

//_______________________________________________
void Application::_configuration( void )
{

    Debug::Throw( "Application::_configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL( configurationChanged() ), SIGNAL( configurationChanged() ) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.toggleSticky( false );
    dialog.exec();

}

//_______________________________________________
void Application::_updateConfiguration( void )
{
    Debug::Throw( "Application::_updateConfiguration.\n" );
    toggleLogAction().setChecked( XmlOptions::get().get<bool>( "LOG_ENABLED" ) );
    hideKilledJobsAction().setChecked( XmlOptions::get().get<bool>( "HIDE_KILLED_JOBS" ) );
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandList();
    if( XmlOptions::get().get<bool>( "SHOW_LOG" ) ) _showLog();
}

//________________________________________________
bool Application::_processCommand( SERVER::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == SERVER::ServerCommand::RAISE )
    {
        mainWindow().uniconify();
        return true;
    } else return false;

}

//_______________________________________________
void Application::_showCommandList( void )
{

    Debug::Throw( "Application::_showCommandList.\n" );
    BASE::KeySet<AlienCommandDialog> dialogs( this );

    if( dialogs.empty() )
    {
        AlienCommandDialog *dialog = new AlienCommandDialog(0);
        BASE::Key::associate( dialog, this );

        dialog->setModel( &_model() );
        dialog->list().setMask( (1<<AlienCommandModel::NAME) );
        dialog->list().setOptionName( "JOB_COMMAND_LIST" );
        dialog->centerOnWidget( qApp->activeWindow() ).show();
        dialog->toggleSticky( false );

        connect( &mainWindow(), SIGNAL( commandRequested( AlienCommand ) ), &dialog->list(), SLOT( resizeColumns( void ) ) );
        connect( &interface(), SIGNAL( commandCompleted( AlienCommand ) ), &dialog->list(), SLOT( resizeColumns( void ) ) );
        connect( dialog, SIGNAL( commandAborted( AlienCommand ) ), &interface(), SLOT( abortCommand( AlienCommand ) ) );
        connect( dialog, SIGNAL( destroyed() ), SLOT( _commandListDestroyed( void ) ) );

        XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );

        return;

    } else {

        (*dialogs.begin())->uniconify();

    }

}

//_______________________________________________
void Application::_showLog( void )
{
    Debug::Throw( "Application::_showLog.\n" );
    logDialog().show();
    logDialog().toggleSticky( false );
    logDialog().raise();
    XmlOptions::get().set<bool>( "SHOW_LOG", true );
}

//_______________________________________________
void Application::_toggleLog( bool value )
{
    Debug::Throw( "Application::_toggleLog.\n" );
    if( value )
    {

        connect( &interface(), SIGNAL( logAvailable( const QString&, unsigned int ) ), &logDialog().editor(), SLOT( append( const QString&, unsigned int ) ), Qt::UniqueConnection );

    } else {

        disconnect( &interface(), SIGNAL( logAvailable( const QString&, unsigned int ) ), &logDialog().editor(), SLOT( append( const QString&, unsigned int ) ) );
        logDialog().hide();
        logDialog().editor().clear();

    }

    logAction().setEnabled( value );
    XmlOptions::get().set<bool>( "LOG_ENABLED", value );
}

//_______________________________________________
void Application::_toggleHideKilledJobs( bool value )
{
    Debug::Throw( "Application::_toggleHideKilledJobs.\n" );
    interface().setHideKilledJobs( value );
    XmlOptions::get().set<bool>("HIDE_KILLED_JOBS", value );
}

//_______________________________________________
void Application::_logDialogHidden( void )
{
    Debug::Throw( "Application::_logDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
}

//_______________________________________________
void Application::_commandListDestroyed( void )
{
    Debug::Throw( "Application::_commandListDestroyed.\n" );
    BASE::KeySet<AlienCommandDialog> dialogs( this );
    if( dialogs.empty() ) XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//_______________________________________________
void Application::_requestJobInformation( const Job& job )
{
    Debug::Throw( "Application::_requestJobInformation\n" );


    // retrieve associated job dialogs
    BASE::KeySet<JobInformationDialog> dialogs( this );

    for( BASE::KeySet<JobInformationDialog>::iterator iter = dialogs.begin(); iter != dialogs.end(); ++iter )
    {
        if( (*iter)->job() == job )
        {
            (*iter)->requestUpdate();
            (*iter)->uniconify();
            return;
        }
    }

    // create dialog
    JobInformationDialog* dialog = new JobInformationDialog( 0 );
    BASE::Key::associate( this, dialog );

    dialog->setJob( job );

    // setup connection
    connect( &interface(), SIGNAL( jobInformationAvailable( const Job&, const QString& ) ), dialog, SLOT( update( const Job&, const QString& ) ) );
    connect( &interface(), SIGNAL( jdlAvailable( const Job&, const QString& ) ), dialog, SLOT( updateJdl( const Job&, const QString& ) ) );
    connect( &interface(), SIGNAL( jobFilesAvailable( const Job&, const QString&, const FileRecord::List& ) ), dialog, SLOT( updateFiles( const Job&, const QString&, const FileRecord::List& ) ) );
    connect( &interface(), SIGNAL( commandCompleted( AlienCommand ) ), dialog, SLOT( commandCompleted( AlienCommand ) ) );
    connect( dialog, SIGNAL( commandRequested( AlienCommand ) ), &interface(), SLOT( addCommand( AlienCommand ) ) );

    // show dialog
    dialog->requestUpdate();
    dialog->centerOnPointer();
    dialog->show();
}

