#ifndef Application_h
#define Application_h

// $Id: Application.h,v 1.14 2012/01/20 17:30:10 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file Application.h
\brief Main Window singleton object
\author Hugo Pereira
\version $Revision: 1.14 $
\date $Date: 2012/01/20 17:30:10 $
*/

#include "BaseApplication.h"

#include "Config.h"
#include "CommandLineArguments.h"
#include "Counter.h"
#include "AlienCommandModel.h"
#include "Key.h"

class MainWindow;
class Job;
class AlienInterface;
class LogDialog;
class ScratchFileMonitor;

//! Main Window singleton object
class Application: public BaseApplication, public BASE::Key, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! command line parser
    static CommandLineParser commandLineParser( CommandLineArguments arguments = CommandLineArguments(), bool ignoreWarnings = true );

    //! usage
    static void usage( void );

    //! constructor
    Application( CommandLineArguments );

    //! destructor
    virtual ~Application( void );

    //! create all widgets
    virtual bool realizeWidget( void );

    //! interface
    virtual AlienInterface& interface( void ) const
    {
        assert( interface_ );
        return *interface_;
    }

    //! retrieves top layout
    virtual MainWindow& mainWindow( void ) const
    {
        assert( mainWindow_ );
        return *mainWindow_;
    }

    //! retrieves top layout
    virtual LogDialog& logDialog( void ) const
    {
        assert( logDialog_ );
        return *logDialog_;
    }

    //! scratch files
    ScratchFileMonitor& scratchFileMonitor( void ) const
    { return *scratchFileMonitor_; }

    //!@name actions
    //@{

    //! command list action
    QAction& commandListAction( void ) const
    { return *commandListAction_; }

    //! log action
    QAction& logAction( void ) const
    { return *logAction_; }

    //! log action
    QAction& toggleLogAction( void ) const
    { return *toggleLogAction_; }

    //! hide killed jobs
    QAction& hideKilledJobsAction( void ) const
    { return *hideKilledJobsAction_; }
    
    //! pause action
    QAction& pauseAction( void ) const
    { return *pauseAction_; }

    //@}

    public slots:

    //! close job dialogs
    virtual void closeJobInformationDialogs( void );

    protected:

    //! install actions
    void _installActions( void );

    //! condor model
    AlienCommandModel& _model( void )
    { return model_; }

    protected slots:

    //! configuration
    virtual void _configuration( void );

    //! about
    virtual void _about( void )
    { BaseApplication::_about( "gridwatch", VERSION, BUILD_TIMESTAMP ); }

    //! process request from application manager
    virtual bool _processCommand( SERVER::ServerCommand );

    //! show command list
    void _showCommandList( void );

    //! show log
    void _showLog( void );

    //! toggle log
    void _toggleLog( bool );

    //! toggle hide killed jobs
    void _toggleHideKilledJobs( bool );

    //! log dialog hidden
    void _logDialogHidden( void );

    //! command list destroyed
    void _commandListDestroyed( void );

    //! request job information
    void _requestJobInformation( const Job& );

    private slots:

    //! configuration
    void _updateConfiguration( void );

    private:

    //! pointer to top interface
    AlienInterface* interface_;

    //! pointer to top widget
    MainWindow *mainWindow_;

    //! command model
    AlienCommandModel model_;

    //! log dialog
    LogDialog* logDialog_;

    //! scratch files
    ScratchFileMonitor* scratchFileMonitor_;

    //! command list
    QAction* commandListAction_;

    //! log action
    QAction* logAction_;

    //! log action
    QAction* toggleLogAction_;

    //! hide killed jobs
    QAction* hideKilledJobsAction_;

    //! pause
    QAction* pauseAction_;

};

#endif
