// $Id: Job.cpp,v 1.24 2011/11/23 11:09:41 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Job.h"
#include "XmlDef.h"

#include <QtCore/QRegExp>

//____________________________________________________________
Job::FieldVector &Job::_fieldNames( void )
{
    static FieldVector singleton( nFields, QString() );
    return singleton;
}

//____________________________________________________________
Job::FieldMap& Job::_fieldMap( void )
{
    static FieldMap singleton;
    return singleton;
}

//____________________________________________________________
void Job::installFields( void )
{
    Debug::Throw( "Job::installFields.\n" );
    _setFieldName( ID, "Id" );
    _setFieldName( USER, "User" );
    _setFieldName( STATUS, "Status" );
    _setFieldName( LOCATION, "Location" );
    _setFieldName( HOST, "Host" );
    _setFieldName( COMMAND, "Command" );
    _setFieldName( SUB_TIME, "Submission");
    _setFieldName( RUN_TIME, "Start");
    _setFieldName( END_TIME, "Completion");
    return;
}

//____________________________________________________________
Job::Job( const QDomElement& element ):
    Counter( "Job" ),
    parentId_( -1 ),
    fields_( nFields )
{

    Debug::Throw( "Job::Job (dom).\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( unsigned int i=0; i<attributes.length(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        if( attribute.name() == XML::PARENT ) setParentId( attribute.value().toInt() );
        else Debug::Throw(0) << "Job::Job (dom) - unrecognized attribute: " << attribute.name() << endl;
    }

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {
        QDomElement childElement = childNode.toElement();
        QString tagName( childElement.tagName() );
        if( tagName == XML::FIELD ) _loadXmlField( childElement );
        else Debug::Throw(0) << "Job::Job - unrecognized child tagName " << tagName << endl;
    }

}

//____________________________________________________________
Job::Job( const QString& buffer ):
    Counter( "Job" ),
    parentId_( -1 ),
    fields_( nFields )
{
    // split buffer
    QStringList fields( buffer.split( QRegExp("\\s+") ) );
    if( fields.size() < 5 )  return;

    assign( USER, fields[0] );

    if( fields.size() == 8 && fields[1] == ".." )
    {

        assign( ID, fields[4] );
        assign( STATUS, fields[6] );
        assign( COMMAND, fields[7] );

    } else if( fields.size() == 7 ) {


        assign( ID, fields[1] );
        assign( LOCATION, fields[3] );
        assign( HOST, fields[4] );
        assign( STATUS, fields[5] );
        assign( COMMAND, fields[6] );

    } else if( fields.size() == 6 ) {

        assign( ID, fields[1] );
        assign( LOCATION, fields[3] );
        assign( STATUS, fields[4] );
        assign( COMMAND, fields[5] );

    } else if( fields.size() == 5 ) {

        assign( ID, fields[1] );
        assign( STATUS, fields[3] );
        assign( COMMAND, fields[4] );

    }

}

//____________________________________________________________
QDomElement Job::domElement( QDomDocument& document ) const
{

    QDomElement out = document.createElement( XML::JOB );
    out.setAttribute( XML::PARENT, QString().setNum( parentId() ) );

    // add fields
    for( unsigned int index=0; index<fields_.size(); ++index )
    {

        if( !has( index ) ) continue;
        QDomElement field = out.appendChild( document.createElement( XML::FIELD ) ).toElement();
        field.setAttribute( XML::FIELD_NAME, _fieldNames()[index] );
        field.appendChild( document.createTextNode( fields_[index] ) );

    }

    return out;

}

//____________________________________________________________
QString Job::status( void ) const
{

    QString st( get( STATUS ) );
    if( id() == 0 ) return "";
    else if( st.isEmpty() ) return "unknown";
    else if( st == "I" ) return "inserting";
    else if( st == "SP" ) return "splitting";
    else if( st == "R" ) return "running";
    else if( st == "D" ) return "completed";
    else if( st == "W" ) return "pending";
    else if( st == "ASSIGNED" ) return "assigned";
    else if( st[0] == 'E' )
    {
        QString buffer;
        QTextStream( &buffer ) << "failed (" << error() << ")";
        return buffer;
    } else if( st == "S" ) return "split";
    else if( st == "SV" ) return "saving";
    else if( st == "SVD" ) return "saved";
    else if( st == "ST" ) return "starting";
    else if( st == "K" ) return "killed";
    else if( st == "Z" ) return "zombie";
    else return st;

}

//____________________________________________________________
QString Job::error( void ) const
{

    QString st( get( STATUS ) );
    if( st.isEmpty() || st[0] != 'E' ) return QString();
    else if( st == "EI" ) return "insertion";
    else if( st == "EA" ) return "assignment";
    else if( st == "ES" ) return "queuing";
    else if( st == "EE" ) return "execution";
    else if( st == "ER" ) return "running";
    else if( st == "EV" ) return "validation";
    else if( st == "EIB" ) return "input box";
    else if( st == "ESV" ) return "saving";
    else if( st == "EXPIRED" ) return "expired";
    else return st;

}

//____________________________________________________________
Job& Job::assign( int index, const Str& value )
{
    _checkIndex(index);
    fields_[index] = value;
    if( index == SUB_TIME ) subTime_ = _time( value );
    else if( index == RUN_TIME ) runTime_ = _time( value );
    else if( index == END_TIME ) endTime_ = _time( value );

    return *this;
}

//____________________________________________________________
void Job::_loadXmlField( const QDomElement& element )
{
    Debug::Throw( "Job::_loadXmlField.\n" );
    QDomNamedNodeMap attributes( element.attributes() );
    int id( 0 );
    for( unsigned int i=0; i<attributes.length(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name != XML::FIELD_NAME )
        {

            Debug::Throw(2) << "Job::_loadXmlField - unrecognized attribute: " << name << endl;
            continue;

        } else if( !_xmlToId( value, id ) ) {

            Debug::Throw(2) << "Job::_loadXmlField - field: " << value << " rejected" << endl;
            return;

        } else {

            Debug::Throw(2) << "Job::_loadXmlField - field: " << value << " accepted" << endl;
            break;

        }
    }

    // get node text and assign
    assign( id, element.text() );

}

//____________________________________________________________
TimeStamp Job::_time( const QString& value )
{

    QRegExp regExp( "(\\w+)\\s+(\\d+)\\s+(\\d+):(\\d+)" );
    const int index( regExp.indexIn( value ) );
    if( index < 0 || regExp.captureCount() < 4 ) return TimeStamp();

    const QStringList captures( regExp.capturedTexts() );
    TimeStamp out( TimeStamp::now() );
    out.setMonth( captures[1] )
        .setDay( captures[2].toInt() )
        .setHours( captures[3].toInt() )
        .setMinutes( captures[3].toInt() ).makeTime();

    return out;
}
