#ifndef Job_h
#define Job_h

// $Id: Job.h,v 1.31 2012/01/19 07:24:54 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*******************************************************************************/


#include "Counter.h"
#include "Debug.h"
#include "JobStatistics.h"
#include "Str.h"
#include "TimeStamp.h"

#include <QtXml/QDomDocument>
#include <QtXml/QDomElement>
#include <QtCore/QString>
#include <QtCore/QString>

#include <cassert>
#include <cmath>
#include <map>
#include <set>
#include <vector>

//! job
class Job: public Counter
{

    public:

    //! number of data fields
    enum { nFields = 9 };

    //! allows fast, keyed access to any data field
    enum FieldType
    {
        ID,
        USER,
        STATUS,
        LOCATION,
        HOST,
        COMMAND,
        SUB_TIME,
        RUN_TIME,
        END_TIME
    };

    //! initialize field names
    static void installFields( void );

    //! get field name
    static const QString& fieldName( unsigned int index )
    {
        assert( index <= _fieldNames().size() );
        return _fieldNames()[index];
    }

    //! empty constructor
    Job( void ):
        Counter( "Job" ),
        parentId_( -1 ),
        fields_( nFields )
    {}

    //: constructor from dom
    Job( const QDomElement& );

    //! constructor from string to browse
    Job( const QString& );

    //! destructor
    virtual ~Job( void )
    {}

    //! dom element
    QDomElement domElement( QDomDocument& ) const;

    //! lower than operator
    bool operator < ( const Job& other ) const
    { return id() < other.id(); }

    //! equal to operator
    bool operator == (const Job& other ) const
    { return id() == other.id(); }

    //! equal to operator
    bool operator != (const Job& other ) const
    { return !(*this == other); }

    //!@name modifiers
    //@{

    //! parent id
    Job& setParentId( const int& value )
    {
        parentId_ = value;
        return *this;
    }

    //! children statistics
    Job& setChildrenStatistics( const JobStatistics& value )
    {
        childrenStatistics_ = value;
        return *this;
    }

    //! set field
    Job& assign( int, const Str& );

    //! set field
    template <typename T>
        Job& assign( int index, const T& value )
    { return assign( index, Str().assign<T>( value ) ); }

    //@}

    //!@name accessors
    //@{

    //! valid
    bool isValid( void ) const
    {
        bool error( false );
        get(ID).get<int>(&error);
        return (!error && has( USER ));
    }

    //! id
    int id( void ) const
    { return has(ID) ? std::abs(get<int>(ID)):-1; }

    //! parent id
    int parentId( void ) const
    { return parentId_; }

    //! returns true job is top level
    bool isMasterJob( void ) const
    { return !( isChild() || hasParent() ); }

    //! returns true if job is a child job
    bool isChild( void ) const
    { return has( ID ) && get<int>( ID )<0; }

    //! returns true if job has a parent
    bool hasParent( void ) const
    { return parentId() > 0; }

    //! returns true if has children
    bool hasChildren( void ) const
    { return childrenStatistics_.get( JobStatistics::TOTAL ) > 0; }

    //! children statistics
    const JobStatistics& childrenStatistics( void ) const
    { return childrenStatistics_; }

    //! returns true if this job is a child of argument
    bool isChild( const Job& job ) const
    { return parentId() == job.id(); }

    //! true if inserting
    bool isInserting( void ) const
    { return get( STATUS ) == "I"; }

    //! true if running
    bool isRunning( void ) const
    {
        return
            get( STATUS ) == "I" ||
            get( STATUS ) == "SP" || // splitting
            get( STATUS ) == "R" ||
            get( STATUS ) == "S" ||
            get( STATUS ) == "ST" ||
            get( STATUS ) == "SV" ||
            get( STATUS ) == "SVD";
    }

    //! true if completed
    bool isCompleted( void ) const
    { return get( STATUS ) == "D"; }

    //! true if pending
    bool isPending( void ) const
    {
        return
            get( STATUS ) == "W" ||
            get( STATUS ) == "ASSIGNED";
    }

    //! true if has error
    bool isFailed( void ) const
    { return has( STATUS ) && ( get( STATUS )[0] == 'E' || get( STATUS ) == "Z" ); }

    //! true if job is killed
    bool isKilled( void ) const
    { return get( STATUS ) == "K"; }

    //! return status
    QString status( void ) const;

    //! return error code
    QString error( void ) const;

    //! const field accessor
    Str get( int index ) const
    {
        _checkIndex(index);
        if( index < nFields ) return fields_[index];
        else return Str().setNum( childrenStatistics_.get( index - nFields ) );
    }

    //! field accessor (cast to anything)
    template <typename T>
        T get( int index ) const
    { return get( index ).get<T>(); }

    //! Check if field makes sence
    bool has( int index ) const
    { return get(index).size(); }

    //! times
    const TimeStamp& subTime( void ) const
    { return subTime_; }

    //! times
    const TimeStamp& runTime( void ) const
    { return runTime_; }

    //! times
    const TimeStamp& endTime( void ) const
    { return endTime_; }

    //@}

    //! used to count total number of jobs
    class TotalFTor
    {
        public:

        //! constructor
        TotalFTor( void )
        {}

        //! destructor
        virtual ~TotalFTor( void )
        {}

        //! predicate
        bool operator() (const Job& job ) const
        {  return !( job.isMasterJob() && job.hasChildren() ); }

    };

    //! used to select pending jobs
    class PendingFTor
    {
        public:

        //! constructor
        PendingFTor( void )
        {}

        //! destructor
        virtual ~PendingFTor( void )
        {}

        //! predicate
        bool operator() (const Job& job ) const
        {  return job.isPending() && !job.hasChildren(); }

    };

    //! used to select running jobs
    class RunningFTor
    {
        public:

        //! constructor
        RunningFTor( void )
        {}

        //! destructor
        virtual ~RunningFTor( void )
        {}

        //! predicate
        bool operator() (const Job& job ) const
        {  return job.isRunning() && !job.hasChildren(); }

    };

    //! used to select completed jobs
    class CompletedFTor
    {
        public:

        //! constructor
        CompletedFTor( void )
        {}

        //! predicate
        bool operator() (const Job& job ) const
        {  return job.isCompleted() && !job.hasChildren(); }

    };

    //! used to select failed jobs
    class FailedFTor
    {
        public:

        //! constructor
        FailedFTor( void )
        {}

        //! destructor
        virtual ~FailedFTor( void )
        {}

        //! predicate
        bool operator() (const Job& job ) const
        {  return job.isFailed() && !job.hasChildren(); }

    };

    //! used to select jobs with given ID
    class SameIdFTor
    {

        public:

        //! constructor
        SameIdFTor( const int& id ):
            id_( id )
        {}

        //! predicate
        bool operator() (const Job& job ) const
        {  return job.id() == id_; }

        private:

        //! field index
        int id_;

    };

    //! shortcut for job id set
    typedef std::set< int > IdSet;

    protected:

    //! set field id from xml field name return false if not found
    static bool _xmlToId( const QString& name, int& id )
    {
        std::map<QString, unsigned int>::iterator iter = _fieldMap().find( name );
        if( iter == _fieldMap().end() ) return false;
        id = iter->second;
        return true;
    }

    //! check index is in bounds
    void _checkIndex( int index ) const
    {
        assert( index >= 0 && index <= nFields + JobStatistics::nFields );
        return;
    }

    //! fields
    typedef std::vector<Str> FieldVector;

    //! field names
    static FieldVector& _fieldNames( void );

    //! load field from DomElement
    void _loadXmlField( const QDomElement& element );

    //! map between xml__fieldNames() and field_id, for fast reading access
    typedef std::map<QString, unsigned int > FieldMap;

    //! xml field map
    static FieldMap& _fieldMap( void );

    //! set xml_field name for given index
    static void _setFieldName( FieldType index, const QString& value )
    {
        _fieldNames()[index] = value;
        _fieldMap().insert( std::make_pair( value, index ) );
    }

    // convert string to time
    TimeStamp _time( const QString& );

    private:

    //! parent id
    /*! it is kept outside of fields in order not to appear in list */
    int parentId_;

    //! fields
    FieldVector fields_;

    //! children statistics
    JobStatistics childrenStatistics_;

    // timestamps
    TimeStamp subTime_;
    TimeStamp runTime_;
    TimeStamp endTime_;

};

#endif
