// $Id: JobManager.cpp,v 1.22 2012/01/20 17:01:36 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    JobManager.cc
\brief   Job managment
\author  Hugo Pereira
\version $Revision: 1.22 $
\date    $Date: 2012/01/20 17:01:36 $
*/

#include "JobManager.h"

#include "AnimatedTreeView.h"
#include "Command.h"
#include "Debug.h"
#include "File.h"
#include "Job.h"
#include "InformationDialog.h"
#include "Str.h"
#include "UserSet.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QtGui/QApplication>
#include <QtGui/QHeaderView>
#include <QtGui/QScrollBar>
#include <QtCore/QTextStream>

#include <set>

//_______________________________________________
JobManager::JobManager( QWidget *parent ):
    QWidget( parent ),
    user_( UserSet::defaultUser() )
{
    Debug::Throw("JobManager::JobManager.\n");

    // layout
    setLayout( new QVBoxLayout( this ) );
    layout()->setSpacing(0);
    layout()->setMargin(0);

    // list
    list_ = new TreeView( this );

    layout()->addWidget( list_ );
    list().setModel( &model_ );
    list().setSortingEnabled( true );
    list().setRootIsDecorated( true );
    list().setSelectionMode( TreeView::ContiguousSelection );
    list().setOptionName( "JOBLIST" );

    connect( &model_, SIGNAL( layoutAboutToBeChanged() ), SLOT( _storeSelectedJobs() ) );
    connect( &model_, SIGNAL( layoutAboutToBeChanged() ), SLOT( _storeExpandedJobs() ) );

    connect( &list(), SIGNAL( expanded( const QModelIndex& ) ), &list(), SLOT( resizeColumns() ) );
    connect( &list(), SIGNAL( collapsed( const QModelIndex& ) ), &list(), SLOT( resizeColumns() ) );

    connect( &model_, SIGNAL( layoutChanged() ), SLOT( _restoreSelectedJobs() ) );
    connect( &model_, SIGNAL( layoutChanged() ), SLOT( _restoreExpandedJobs() ) );

}

//_______________________________________________
JobManager::~JobManager( void )
{ Debug::Throw( "JobManager::~JobManager.\n"); }

//_______________________________________
bool JobManager::setUser( QString user )
{

    Debug::Throw() << "JobManager::setUser - " << user << endl;
    if( user.isEmpty() || user == user_ ) return false;
    user_ = user;
    model_.clear();
    return true;

}

//____________________________________________________________
JobSet JobManager::updateJobList( JobSet jobs )
{
    Debug::Throw( "JobManager::updateJobList.\n" );

    // retrieve current set
    JobModel::List currentJobs( model_.children() );

    // loop over
    if( !currentJobs.empty() )
    {

        emit busy( currentJobs.size() );
        int index(0);

        for( JobModel::List::const_iterator currentIter = currentJobs.begin(); currentIter != currentJobs.end(); ++currentIter, ++index )
        {

            emit progressAvailable( index );

            // skip if timestamps are not set
            if( !( currentIter->has( Job::SUB_TIME ) || currentIter->has( Job::RUN_TIME ) || currentIter->has( Job::END_TIME ) || currentIter->hasParent() ) )
            { continue; }

            // find in current job set
            JobSet::const_iterator iter( jobs.find( *currentIter ) );
            if( iter != jobs.end() )
            {
                Job job( *iter );
                bool changed( false );

                // copy parent if needed
                if( currentIter->hasParent() && currentIter->parentId() != iter->parentId() ) { changed = true; job.setParentId( currentIter->parentId() ); }

                // copy timestamps if needed
                if( currentIter->has( Job::SUB_TIME ) && !iter->has( Job::SUB_TIME ) ) { changed = true; job.assign( Job::SUB_TIME, currentIter->get( Job::SUB_TIME ) ); }
                if( currentIter->has( Job::RUN_TIME ) && !iter->has( Job::RUN_TIME ) ) { changed = true; job.assign( Job::RUN_TIME, currentIter->get( Job::RUN_TIME ) ); }
                if( currentIter->has( Job::END_TIME ) && !iter->has( Job::END_TIME ) ) { changed = true; job.assign( Job::END_TIME, currentIter->get( Job::END_TIME ) ); }

                if( changed )
                {
                    jobs.erase( *iter );
                    jobs.insert( job );
                }

            }

        }

    }

    // keep track of jobs that are missing time stamps
    JobSet out;
    for( JobSet::const_iterator iter = jobs.begin(); iter != jobs.end(); ++iter )
    {

        // skip killed jobs, since they will dissapear
        if( iter->isKilled() ) continue;

        // check for missing submission time
        if( !iter->has( Job::SUB_TIME ) ) out.insert( *iter );

        // check for missing run time
        if( ( ( iter->isRunning() && !iter->isInserting() ) || iter->isCompleted() || iter->isFailed() ) && !iter->has( Job::RUN_TIME ) ) out.insert( *iter );

        // check for missing end time
        else if( ( iter->isCompleted() || iter->isFailed() ) && !iter->has( Job::END_TIME ) ) out.insert( *iter );

    }

    // assign to model
    model_.set( jobs );

    // iddle
    emit idle();

    QString buffer;
    QTextStream( &buffer ) << "Last update: " << TimeStamp::now().toString( TimeStamp::TIME_LONG );
    emit messageAvailable( buffer );

    return out;
}

//____________________________________________________________
void JobManager::updateJob( const Job& job )
{

    Debug::Throw() << "JobManager::updateJob - job " << job.id() << endl;

    // get index from job
    QModelIndex index( model_.index( job ) );
    if( !index.isValid() ) return;

    // get job
    Job local( model_.get( index ) );
    if( job.has( Job::SUB_TIME ) ) local.assign( Job::SUB_TIME, job.get( Job::SUB_TIME ) );
    if( job.has( Job::RUN_TIME ) ) local.assign( Job::RUN_TIME, job.get( Job::RUN_TIME ) );
    if( job.has( Job::END_TIME ) ) local.assign( Job::END_TIME, job.get( Job::END_TIME ) );

    if( job.hasParent() ) local.setParentId( job.parentId() );

    // re-insert in model
    model_.add( local );

}

//____________________________________________________________
void JobManager::updateJobChildren( const Job& job, const Job::IdSet& children )
{

    Debug::Throw() << "JobManager::updateJobChildren - job " << job.id() << endl;

    // retrieve current set
    JobSet updatedJobs;

    emit busy( children.size() );
    int index(0);

    for( Job::IdSet::const_iterator iter = children.begin(); iter != children.end(); ++iter, ++index )
    {

        emit progressAvailable( index );

        Job child;
        child.assign( Job::ID, QString().setNum( *iter ) );

        // find in model
        QModelIndex index( model_.index( child ) );
        if( !index.isValid() ) continue;

        // get full job
        Job local( model_.get( index ) );

        // check parent and update if found
        if( local.parentId() != job.id() )
        {
            local.setParentId( job.id() );
            updatedJobs.insert( local );
        }
    }

    // update model
    model_.add( updatedJobs );

    // iddle
    emit idle();

}

//________________________________________
JobSet JobManager::selectedJobs( bool check )
{
    Debug::Throw( "JobManager::selectedJobs.\n" );
    JobSet selected;

    // retrieve selected indexes in list
    QModelIndexList selectedIndexes( list().selectionModel()->selectedIndexes() );
    for( QModelIndexList::iterator iter = selectedIndexes.begin(); iter != selectedIndexes.end(); ++iter )
    {

        // check column
        if( !iter->column() == 0 ) continue;
        if( !list().isVisible( *iter ) ) continue;

        // get current job
        const Job& job( model_.get( *iter ) );

        selected.insert( job );

    }

    if( check && selected.empty() )
    {
        // popup warning when no job is selected (nor removed by "permission denied" )
        QString buffer( "No selected jobs." );
        InformationDialog( this, buffer ).exec();
    }

    return selected;

}

//________________________________________
void JobManager::_storeSelectedJobs( void )
{

    // clear
    model_.clearSelectedIndexes();

    // retrieve selected indexes in list
    QModelIndexList selectedIndexes( list().selectionModel()->selectedIndexes() );
    for( QModelIndexList::iterator iter = selectedIndexes.begin(); iter != selectedIndexes.end(); ++iter )
    {
        // check column
        if( !iter->column() == 0 ) continue;
        model_.setIndexSelected( *iter, true );
    }

}

//________________________________________
void JobManager::_restoreSelectedJobs( void )
{

    QModelIndexList selectedIndexes( model_.selectedIndexes() );
    if( selectedIndexes.empty() ) list().selectionModel()->clear();
    else {

        list().selectionModel()->select( selectedIndexes.front(),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        for( QModelIndexList::const_iterator iter = selectedIndexes.begin(); iter != selectedIndexes.end(); ++iter )
        { list().selectionModel()->select( *iter, QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

    }

    return;
}


//________________________________________
void JobManager::_storeExpandedJobs( void )
{

    // clear
    model_.clearExpandedIndexes();

    // retrieve selected indexes in list
    QModelIndexList indexes( model_.indexes() );
    for( QModelIndexList::iterator iter = indexes.begin(); iter != indexes.end(); ++iter )
    { if( list().isExpanded( *iter ) ) model_.setIndexExpanded( *iter, true ); }

    list().storeScrollBarPosition();

}

//________________________________________
void JobManager::_restoreExpandedJobs( void )
{

    QModelIndexList expandedIndexes( model_.expandedIndexes() );
    list().collapseAll();
    for( QModelIndexList::const_iterator iter = expandedIndexes.begin(); iter != expandedIndexes.end(); ++iter )
    { list().setExpanded( *iter, true ); }

    list().restoreScrollBarPosition();
}

