// $Id: MainWindow.cpp,v 1.52 2012/01/20 17:01:36 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file MainWindow.cc
\brief main application widget
\author Hugo Pereira
\version $Revision: 1.52 $
\date $Date: 2012/01/20 17:01:36 $
*/

#include "MainWindow.h"

#include "AnimatedTreeView.h"
#include "Application.h"
#include "CustomToolBar.h"
#include "File.h"
#include "Icons.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "JobCollection.h"
#include "JobManager.h"
#include "JobManagerFrame.h"
#include "JobSet.h"
#include "JobStatistics.h"
#include "JobStatusBar.h"
#include "Menu.h"
#include "NewFrameDialog.h"
#include "QuestionDialog.h"
#include "QtUtil.h"
#include "SummaryFrame.h"
#include "Singleton.h"
#include "UserSelectionFrame.h"
#include "UserSet.h"
#include "Util.h"
#include "XmlDef.h"
#include "XmlOptions.h"

#include <QtCore/QTextStream>

//___________________________________________
void TOP::TabWidget::tabInserted( int index )
{
    AnimatedTabWidget::tabInserted( index );
    int count( QTabWidget::count() );
    if( tabBar() ) tabBar()->setVisible( count > 1 );
}

//___________________________________________
void TOP::TabWidget::tabRemoved( int index )
{
    AnimatedTabWidget::tabRemoved( index );
    int count( QTabWidget::count() );
    if( tabBar() ) tabBar()->setVisible( count > 1 );
}

//___________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    activeFrame_( 0 ),
    refreshRate_( 0 ),
    maxTracedJobs_( 0 )
{
    Debug::Throw( "MainWindow::MainWindow.\n" );

    setOptionName( "MAIN_WINDOW" );

    // install actions
    _installActions();
    Application& application( *static_cast<Application*>(Singleton::get().application()) );
    addAction( &application.closeAction() );

    // statusBar
    setStatusBar( statusbar_ = new JobStatusBar( this ) );

    // toolbar
    userToolBar_ =  new CustomToolBar( "User Toolbar", this, "USER_TOOLBAR" );
    userToolBar_->addWidget( userSelectionFrame_ = new UserSelectionFrame( userToolBar_ ) );

    // add default users to user_selection frame
    userSelectionFrame().updateUsers( UserSet() );

    toolbar_ =  new CustomToolBar( "Main Toolbar", this, "TOOLBAR" );
    toolbar_->addAction( &addFrameAction() );
    toolbar_->addSeparator();
    toolbar_->addAction( &resubmitAction() );
    toolbar_->addAction( &killAction() );
    toolbar_->addAction( &traceAction() );
    toolbar_->addSeparator();
    toolbar_->addAction( &updateAction() );
    toolbar_->addAction( &clearAction() );

    // for debugging only
    // toolbar_->addAction( &application.pauseAction() );

    // main widget and layout
    QWidget* main( new QWidget( this ) );
    main->setLayout( new QVBoxLayout() );
    main->layout()->setMargin(0);
    main->layout()->setSpacing(0);
    setCentralWidget( main );

    // tab widget
    main->layout()->addWidget( tabWidget_ = new TOP::TabWidget( main ) );
    _tabWidget().setMovable( true );

    connect( &_tabWidget(), SIGNAL( currentChanged( int ) ), SLOT( _activeFrameChanged( int ) ) );

    // create removeTab button
    {
        QToolButton* button = new QToolButton();
        button->setDefaultAction( &removeFrameAction() );
        button->setAutoRaise( false );
        _tabWidget().setCornerWidget( button, Qt::TopRightCorner );
    }

    // menu bar
    setMenuBar( new Menu( this ) );
    connect( &userSelectionFrame(), SIGNAL( userChanged( QString ) ), SLOT( _updateUser( QString ) ) );

    // configuration update
    connect( &application, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    connect( &application, SIGNAL( saveConfiguration() ), SLOT( _saveConfiguration() ) );
    connect( qApp, SIGNAL( aboutToQuit() ), SLOT( _saveConfiguration() ) );

    _updateConfiguration();

}

//___________________________________________
MainWindow::~MainWindow( void )
{ Debug::Throw( "MainWindow::~MainWindow.\n" ); }

//____________________________________________________________
void MainWindow::requestDefaultUser( void )
{
    Debug::Throw( "MainWindow::requestDefaultUser.\n" );
    emit commandRequested( AlienCommand( AlienCommand::DEFAULT_USER ) );
}

//____________________________________________________________
void MainWindow::requestUserList( void )
{
    Debug::Throw( "MainWindow::requestUserList.\n" );
    emit commandRequested( AlienCommand( AlienCommand::USER_LIST ) );
}

//____________________________________________________________
void MainWindow::requestJobList( void )
{
    Debug::Throw( "MainWindow::requestJobList.\n" );

    // store managers that must be updated
    std::vector<JobManager*> managers;

    // start with active manager
    if( _hasActiveFrame() ) managers.push_back( &_activeFrame().jobManager() );

    // add managers for which summary is detached
    BASE::KeySet<JobManagerFrame> frames( this );
    for( BASE::KeySet<JobManagerFrame>::const_iterator iter = frames.begin(); iter != frames.end(); ++iter )
    {

        // check frame
        JobManagerFrame& frame( **iter );
        if( !frame.isDetached() ) continue;
        if( _isActiveFrame( frame ) ) continue;

        // append
        managers.push_back( &frame.jobManager() );

    }

    // add corresponding users as argument
    for( std::vector<JobManager*>::const_iterator iter = managers.begin(); iter != managers.end(); ++iter )
    { _requestJobList( (*iter)->user() ); }

}

//____________________________________________________________
void MainWindow::updateDefaultUser( QString user )
{
    Debug::Throw() << "MainWindow::updateDefaultUser - user: " << user << endl;
    defaultUser_ = user;

    // update frame names
    BASE::KeySet<JobManagerFrame> frames( this );
    for( BASE::KeySet<JobManagerFrame>::const_iterator iter = frames.begin(); iter != frames.end(); ++iter )
    {
        JobManagerFrame& frame( **iter );
        if( UserSet::isDefaultUser( frame.jobManager().user() ) )
        { frame.setUser( user ); }
    }

    // update user selection frame
    if( UserSet::isDefaultUser( userSelectionFrame().user() ) )
    { userSelectionFrame().setUser( defaultUser_ ); }

    // request job list
    requestJobList();

}

//____________________________________________________________
void MainWindow::processUserList( UserSet users )
{

    Debug::Throw( "MainWindow::processUserList.\n" );

    if( _hasDefaultUser() ) users.setNeedsDefault( false );
    userSelectionFrame().updateUsers( users );

    // install frames
    if( BASE::KeySet<JobManagerFrame>(this).empty() ) installFrames();

    // need to reemit the signal so that it gets catched
    // by possible newFrame dialogs
    emit userListAvailable( users );
}

//____________________________________________________________
void MainWindow::processJobList( QString user, const JobSet& jobs )
{

    JobManagerFrame* frame( _findFrame( user ) );
    if( !frame ) return;

    const JobSet out( frame->jobManager().updateJobList( jobs ) );
    frame->summaryFrame().updateJobStatistics( JobStatistics( jobs ) );

    // update clear action
    if( _isActiveFrame( *frame ) )
    {

        // update clear action
        _updateClearAction();

        // submit low priority TRACE commands
        // first split set into master and children jobs
        JobSet masterJobs;
        JobSet childrenJobs;

        int index(0);
        for( JobSet::const_iterator iter = out.begin(); iter != out.end() && (maxTracedJobs_ <= 0 || index < maxTracedJobs_); ++iter )
        {
            if( iter->isChild() ) { childrenJobs.insert( *iter ); }
            else { masterJobs.insert( *iter ); ++index; }
        }

        // insert master jobs
        index = 0;
        for( JobSet::const_iterator iter = masterJobs.begin(); iter != masterJobs.end() && (maxTracedJobs_ <= 0 || index < maxTracedJobs_); ++iter )
        {

            AlienCommand command( AlienCommand::TRACE );
            command
                << user
                << QString().setNum(iter->parentId() )
                << QString().setNum(iter->id() );
            command.setPriority(1);
            emit commandRequested( command );
            ++index;
        }

        // first insert children jobs
        for( JobSet::const_iterator iter = childrenJobs.begin(); iter != childrenJobs.end() && (maxTracedJobs_ <= 0 || index < maxTracedJobs_); ++iter )
        {

            AlienCommand command( AlienCommand::TRACE );
            command
                << user
                << QString().setNum(iter->parentId() )
                << QString().setNum(iter->id() );
            command.setPriority(1);
            emit commandRequested( command );
            ++index;
        }

    }

    // update actions
    _updateActions();

    return;
}

//____________________________________________________________
void MainWindow::processJobInformation( const Job& job )
{

    Debug::Throw() << "MainWindow::processJobInformation - job: " << job.id() << " user: " << job.get( Job::USER ) << endl;

    // find matching frame and update
    if( JobManagerFrame* frame = _findFrame( job.get( Job::USER ) ) )
    { frame->jobManager().updateJob( job ); }

}

//____________________________________________________________
void MainWindow::processJobChildren( const Job& job, const Job::IdSet& children )
{

    Debug::Throw() << "MainWindow::processJobChildren - job: " << job.id() << " user: " << job.get( Job::USER ) << endl;

    // find matching frame and update
    if( JobManagerFrame* frame = _findFrame( job.get( Job::USER ) ) )
    { frame->jobManager().updateJobChildren( job, children ); }

}

//____________________________________________________________
void MainWindow::commandCompleted( AlienCommand command )
{
    switch( command.type() )
    {

        // trigger job list update for kill and resubmit commands
        case AlienCommand::KILL:
        case AlienCommand::KILL_MASTER:
        case AlienCommand::RESUBMIT:
        case AlienCommand::RESUBMIT_MASTER:
        {
            // get user
            const QString user( command.arguments()[0] );

            // decide whether JobList update is needed
            bool updateNeeded( false );
            if( _hasActiveFrame() && _activeFrame().jobManager().user() == user ) updateNeeded = true;
            else {

                // loop over associated JobManagerFrames, try find one with matching user which is detached.
                // trigger update if found.
                BASE::KeySet<JobManagerFrame> frames( this );
                for( BASE::KeySet<JobManagerFrame>::const_iterator iter = frames.begin(); iter != frames.end(); ++iter )
                {

                    // check frame
                    JobManagerFrame& frame( **iter );
                    if( frame.isDetached() && frame.jobManager().user() == user )
                    {
                        updateNeeded = true;
                        break;
                    }

                }

            }

            if( updateNeeded ) _requestJobList( user );

        }

        // do nothing for all other commands
        default: break;
    }

}

//_______________________________________________
bool MainWindow::installToolBarsActions( QMenu& menu )
{
    Debug::Throw( "MainWindow::installToolBarsActions.\n" );
    bool out( BaseMainWindow::installToolBarsActions( menu ) );
    menu.addAction( &toggleSummaryAction() );
    return out;
}

//_______________________________________
void MainWindow::installFrames( void )
{

    Debug::Throw( "MainWindow::installFrames.\n" );

    JobCollection collection;
    collection.read( File( XmlOptions::get().raw( "XML_FILE" ) ) );

    JobManagerFrame* firstFrame(0);
    const JobCollection::List& jobSets( collection.jobSets() );
    if( jobSets.empty() )
    {

        // create new Frame
        JobManagerFrame& frame( _newFrame() );
        if( !firstFrame ) firstFrame = &frame;
        frame.setUser( UserSet::defaultUser() );
        _tabWidget().addTab( &frame, frame.jobManager().user() );

    } else for( JobCollection::List::const_iterator iter = jobSets.begin(); iter != jobSets.end(); ++iter ) {

        // create new Frame
        JobManagerFrame& frame( _newFrame() );
        if( iter->isActive() || !firstFrame ) firstFrame = &frame;

        // assign user
        frame.setUser( iter->owner() );

        // assign jobs
        frame.jobManager().updateJobList( *iter );
        frame.summaryFrame().updateJobStatistics( JobStatistics( *iter ) );

        // add to tab widget
        _tabWidget().addTab( &frame, frame.jobManager().user() );

    }

    assert( firstFrame );
    _setActiveFrame( *firstFrame );
    _tabWidget().setCurrentWidget( firstFrame );

}

//_______________________________________
void MainWindow::_updateConfiguration( void )
{
    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // needed to get proper size (from options) at startup
    resize( sizeHint() );

    // toggle summary
    toggleSummaryAction().setChecked( XmlOptions::get().get<bool>( "SHOW_SUMMARY" ) );

    // refresh rate
    refreshRate_ = 1000 * XmlOptions::get().get<int>( "REFRESH_RATE" );

    // max traced jobs
    maxTracedJobs_ = XmlOptions::get().get<int>( "MAX_TRACED_JOBS" );

    // restart timer
    _timer().start( refreshRate_, this );

}

//________________________________________
void MainWindow::_saveConfiguration( void )
{
    Debug::Throw( "MainWindow::_saveConfiguration.\n" );

    // create job collection
    JobCollection collection;

    // add job sets from existing frames
    BASE::KeySet<JobManagerFrame> frames( this );
    for( BASE::KeySet<JobManagerFrame>::const_iterator iter = frames.begin(); iter != frames.end(); ++iter )
    {
        // local copies
        JobManagerFrame& frame( **iter );
        JobManager& manager( frame.jobManager() );

        // construct job set
        JobSet jobs;
        jobs.setOwner( manager.user() );
        jobs.setActive( _isActiveFrame( frame ) );
        const JobModel::List jobList( manager.model().children() );
        jobs.insert( jobList.begin(), jobList.end() );

        // add to collection
        collection.add( jobs );

    }

    collection.write( File( XmlOptions::get().raw( "XML_FILE" ) ) );

}

//_______________________________________
void MainWindow::_updateActions( void )
{

    Debug::Throw( "MainWindow::_updateActions.\n" );

    // retrace action
    traceAction().setEnabled( _hasActiveFrame() && _activeFrame().jobManager().list().selectionModel()->currentIndex().isValid() );

    // resubmit and kill actions
    if( !_activeFrameIsDefault() )
    {

        resubmitAction().setEnabled( false );
        killAction().setEnabled( false );
        return;

    } else {

        // get list of selected jobs
        const JobSet selectedJobs( _activeFrame().jobManager().selectedJobs() );
        killAction().setEnabled( !selectedJobs.empty() );

        bool resubmitEnabled( false );
        for( JobSet::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
        {

            // trigger on either failed jobs, or master jobs with failed children
            if( (!iter->isCompleted()) || ( iter->isMasterJob() && iter->childrenStatistics().get( JobStatistics::FAILED ) > 0 ) )
            {
                resubmitEnabled = true;
                break;
            }

        }

        // enable action
        resubmitAction().setEnabled( resubmitEnabled );
        return;
    }

}

//___________________________________________________________
void MainWindow::_updateClearAction( void )
{
    Debug::Throw( "MainWindow::_updateClearAction.\n" );
    if( !_activeFrameIsDefault() )
    {

        Debug::Throw( "MainWindow::_updateClearAction - invalid user\n" );
        clearAction().setEnabled( false );
        return;

    } else {

        const JobModel::List jobs( _activeFrame().jobManager().model().children() );
        const bool hasCompletedJobs( std::find_if(  jobs.begin(), jobs.end(), Job::CompletedFTor() ) != jobs.end() );
        Debug::Throw() << "MainWindow::_updateClearAction - completed jobs:" << hasCompletedJobs << endl;

        clearAction().setEnabled( hasCompletedJobs );

    }

}

//___________________________________________________________
void MainWindow::_updateUser( QString user )
{

    Debug::Throw() << "MainWindow::_updateUser - " << user << endl;
    if( !_hasActiveFrame() ) return;

    // automatically replace user name if default
    if( UserSet::isDefaultUser( user ) && _hasDefaultUser() ) user = defaultUser_;

    // check if there is a matching frame
    if( JobManagerFrame* frame = _findFrame( user ) )
    {

        _tabWidget().setCurrentWidget( frame );
        _setActiveFrame( *frame );

    } else {

        _tabWidget().setTabText( _tabWidget().currentIndex(), user );
        _activeFrame().setUser( user );

    }

    // notify change of active users
    emit activeUsersChanged( _users() );

    // request job list
    requestJobList();

}

//_______________________________________
void MainWindow::_toggleSummary( bool value )
{

    Debug::Throw( "MainWindow::_toggleSummary.\n" );
    if( !_hasActiveFrame() ) return;
    _activeFrame().summaryFrame().setVisible( value );
    XmlOptions::get().set<bool>( "SHOW_SUMMARY", value );

}

//_______________________________________
void MainWindow::_activeFrameChanged( int index )
{

    Debug::Throw() << "MainWindow::_activeFrameChanged - index: " << index << endl;
    assert( index >= 0 );
    JobManagerFrame* frame = qobject_cast<JobManagerFrame*>( _tabWidget().widget( index ) );
    assert( frame );

    // change frame activity and update
    _setActiveFrame( *frame );

    // update userSelectionFrame so that it displays the correct user name
    userSelectionFrame().setUser( frame->jobManager().user() );

    // request new job list
    _requestJobList( frame->jobManager().user() );

    // update title
    QString buffer;
    QTextStream( &buffer ) << frame->jobManager().user() << " - gridwatch";
    setWindowTitle( buffer );

}

//_______________________________________
void MainWindow::_addFrame( void )
{
    Debug::Throw( "MainWindow::_addFrame.\n" );

    // create dialog
    NewFrameDialog dialog( this );

    // update user selection
    dialog.userSelectionFrame().updateUsers( userSelectionFrame().users() );
    dialog.userSelectionFrame().setUser( userSelectionFrame().user() );

    // connect for auto-update
    connect(
        this,
        SIGNAL( userListAvailable( std::set<QString> ) ),
        &dialog.userSelectionFrame(),
        SLOT( updateUsers( std::set<QString> ) ) );

    requestUserList();

    // adjust focus and map dialog
    dialog.userSelectionFrame().editor().lineEdit()->setFocus();
    dialog.userSelectionFrame().editor().lineEdit()->selectAll();
    if( !dialog.centerOnWidget( window() ).exec() ) return;

    // get user
    QString user( dialog.userSelectionFrame().user() );
    if( UserSet::isDefaultUser( user ) && _hasDefaultUser() ) user = defaultUser_;

    // check whether it is already displayed or not
    if( JobManagerFrame *frame = _findFrame( user ) )
    {

        _tabWidget().setCurrentWidget( frame );

    } else {

        // create new Frame
        JobManagerFrame& frame( _newFrame() );
        frame.setUser( user );

        _tabWidget().addTab( &frame, user );
        _tabWidget().setCurrentWidget( &frame );

    }

    // notify change of active users
    emit activeUsersChanged( _users() );

    // request job list
    _requestJobList( user );

}


//_______________________________________
void MainWindow::_removeFrame( void )
{
    Debug::Throw( "MainWindow::_removeFrame.\n" );

    unsigned int counts( BASE::KeySet<JobManagerFrame>( this ).size() );
    assert( counts > 1 && _hasActiveFrame() );

    QString buffer;
    QTextStream( &buffer, QIODevice::WriteOnly ) << "Remove tab \"" << _activeFrame().jobManager().user() << "\" ?";
    if( !QuestionDialog( this, buffer ).centerOnWidget( window() ).exec() ) return;

    // change remove action enability
    if( counts == 2 ) removeFrameAction().setEnabled( false );

    // store current index and frame locally
    int index( _tabWidget().currentIndex() );
    JobManagerFrame* frame( &_activeFrame() );

    // change active frame
    _tabWidget().setCurrentIndex( ( index > 0 ) ? index-1:index );
    _tabWidget().removeTab( index );

    // delete old active frame
    frame->deleteLater();

    // notify change of active users
    emit activeUsersChanged( _users() );

}

//____________________________________________________________
void MainWindow::_resubmit( void )
{
    Debug::Throw( "MainWindow::_resubmit.\n" );
    if( !_hasActiveFrame() ) return;

    // check user
    if( !_activeFrameIsDefault() ) return;

    const JobSet selectedJobs( _activeFrame().jobManager().selectedJobs() );
    const QString user( _activeFrame().jobManager().user() );

    // store list of resubmitted master jobs
    JobSet masterJobs;

    // list of states to be checked
    typedef std::set<QString> StringSet;

    // first path to select master jobs
    for( JobSet::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
    {

        // skip if not master
        if( !iter->isMasterJob() ) continue;

        // retrieve children from model and check wether at least one is failed
        const JobModel::List children( _activeFrame().jobManager().model().children( *iter ) );

        // list of states to be checked
        StringSet statusSet;

        // keep track of whether master job has failed children
        for( JobModel::List::const_iterator childIter = children.begin(); childIter != children.end(); ++childIter )
        {
            if( !childIter->isFailed() ) continue;
            statusSet.insert( childIter->get( Job::STATUS ) );
        }

        // skip if no invalid status is found
        if( statusSet.empty() ) continue;

        // submit commands
        for( StringSet::const_iterator statusIter = statusSet.begin(); statusIter != statusSet.end(); ++statusIter )
        {
            QString status( *statusIter );

            // fix status string
            if( status == "Z" ) status = "ZOMBIE";
            else if( !status.isEmpty() && status[0] == 'E' && status.indexOf( "ERROR_" ) < 0 && status != "EXPIRED" )
            { status.replace( QRegExp( "^E" ), "ERROR_" ); }

            // emit command
            emit commandRequested( AlienCommand( AlienCommand::RESUBMIT_MASTER ) << user << QString().setNum(iter->id()) << status );
        }

        // also resubmit trace command, to re-parse parents
        emit commandRequested( AlienCommand( AlienCommand::TRACE ) << user
            << QString().setNum(iter->parentId() )
            << QString().setNum(iter->id()) );

        // insert in master list
        masterJobs.insert( *iter );

    }

    // second path to select subJobs
    // keep track of running jobs
    JobSet runningJobs;
    for( JobSet::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
    {

        // skip master jobs
        if( iter->isMasterJob() ) continue;

        // skip if parent is already in master list
        if( masterJobs.find( Job().assign<int>( Job::ID, iter->parentId() ) ) != masterJobs.end() ) continue;

        if( iter->isFailed() )
        {

            // submit command
            emit commandRequested( AlienCommand( AlienCommand::RESUBMIT ) << user << QString().setNum(iter->id()) );

        } else if( !iter->isCompleted() ) {

            runningJobs.insert( *iter );

        }

    }

    // ask confirmation for running jobs
    if( !runningJobs.empty() )
    {
        QString buffer;
        QTextStream what( &buffer );
        if( runningJobs.size() == 1 ) what << "Job " << runningJobs.begin()->id() << " is";
        else {
            what << "Jobs ";
            for( JobSet::const_iterator iter = runningJobs.begin(); iter != runningJobs.end(); iter++ )
            { what << iter->id() << " "; }

            what << "are ";

        }

        what << "either pending or allready running." << endl;
        what << "Resubmit anyway ?";
        if( QuestionDialog( this, buffer ).centerOnWidget( window() ).exec() )
        {
            for( JobSet::const_iterator iter = runningJobs.begin(); iter != runningJobs.end(); iter++ )
            { emit commandRequested( AlienCommand( AlienCommand::RESUBMIT ) << user << QString().setNum(iter->id()) ); }
        }

    }

}

//____________________________________________________________
void MainWindow::_requestJobInformation( void )
{
    Debug::Throw( "MainWindow::_requestJobInformation.\n" );
    if( !_hasActiveFrame() ) return;

    // get current Job
    QModelIndex index( _activeFrame().jobManager().list().selectionModel()->currentIndex() );
    if( !index.isValid() ) return;

    // emit signal
    emit jobInformationRequested( _activeFrame().jobManager().model().get( index ) );

}

//____________________________________________________________
void MainWindow::_kill( void )
{
    Debug::Throw( "MainWindow::kill.\n" );
    if( !_hasActiveFrame() ) return;

    const JobSet selectedJobs( _activeFrame().jobManager().selectedJobs() );
    JobSet masterJobs;

    // store user
    const QString user( _activeFrame().jobManager().user() );

    // first path to select master jobs
    for( JobSet::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
    {

        // skip if not master
        if( !iter->isMasterJob() ) continue;

        emit commandRequested( AlienCommand( AlienCommand::KILL_MASTER ) << user << QString().setNum(iter->id()) );
        emit commandRequested( AlienCommand( AlienCommand::KILL ) << user << QString().setNum(iter->id()) );

        // insert in master list
        masterJobs.insert( *iter );

    }

    // second path to select subJobs
    for( JobSet::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
    {

        // skip master jobs
        if( iter->isMasterJob() ) continue;

        // skip if parent is already in master list
        if( masterJobs.find( Job().assign<int>( Job::ID, iter->parentId() ) ) != masterJobs.end() ) continue;
        emit commandRequested( AlienCommand( AlienCommand::KILL ) << user << QString().setNum(iter->id()) );

    }

}

//____________________________________________________________
void MainWindow::_clear( void )
{
    Debug::Throw( "MainWindow::clear.\n" );
    if( !_activeFrameIsDefault() ) return;

    // retrieve children from model and check wether at least one is failed
    const JobModel::List selectedJobs( _activeFrame().jobManager().model().children() );
    JobSet masterJobs;

    // store user
    const QString user( _activeFrame().jobManager().user() );

    // first path to select master jobs
    for( JobModel::List::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
    {

        // skip if not master
        if( !iter->isMasterJob() ) continue;

        // retrieve children from model and check wether at least one is completed
        JobModel::List children( _activeFrame().jobManager().model().children( *iter ) );
        const unsigned int completedJobs( std::count_if( children.begin(), children.end(), Job::CompletedFTor() ) );
        if( completedJobs > 0 )
        {

            // insert in list to avoid duplicates in second path
            masterJobs.insert( *iter );

            // kill all DONE children
            emit commandRequested( AlienCommand( AlienCommand::KILL_MASTER ) << user << QString().setNum(iter->id()) << "DONE" );

            // kill the job itself, if tagged completed and all subjobs are completed
            if( iter->isCompleted() && completedJobs == children.size() )
            { emit commandRequested( AlienCommand( AlienCommand::KILL ) << user << QString().setNum(iter->id()) ); }

        }

    }

    // second path to select other jobs
    for( JobModel::List::const_iterator iter = selectedJobs.begin(); iter != selectedJobs.end(); ++iter )
    {

        // skip master jobs
        if( iter->isMasterJob() ) continue;

        // skip if not failed
        if( !iter->isCompleted() ) continue;

        // skip if parent is already in master list
        if( masterJobs.find( Job().assign<int>( Job::ID, iter->parentId() ) ) != masterJobs.end() ) continue;

        // insert in sublist
        emit commandRequested( AlienCommand( AlienCommand::KILL ) << user << QString().setNum(iter->id()) );

    }

}

//____________________________________
void MainWindow::timerEvent( QTimerEvent *event )
{

    Debug::Throw( "MainWindow::timerEvent.\n" );
    if (event->timerId() == timer_.timerId() )
    { requestJobList(); }

}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    _saveConfiguration();
    event->accept();
    qApp->quit();
}

//____________________________________________________________
JobManagerFrame* MainWindow::_findFrame( const QString& user ) const
{

    BASE::KeySet<JobManagerFrame> frames( this );
    for( BASE::KeySet<JobManagerFrame>::const_iterator iter = frames.begin(); iter != frames.end(); ++iter )
    {

        JobManagerFrame* frame( *iter );
        QString frameUser( frame->jobManager().user() );
        if( UserSet::isDefaultUser( frameUser ) ) frameUser = defaultUser_;
        if( frameUser == user ) return frame;
    }

    return 0;

}

//____________________________________________________________
bool MainWindow::_activeFrameIsDefault( void ) const
{ return _hasActiveFrame() && _hasDefaultUser() && _activeFrame().jobManager().user() == defaultUser_; }

//____________________________________________________________
void MainWindow::_requestJobList( QString user )
{

    Debug::Throw() << "MainWindow::_requestJobList - user: " << user << endl;
    if( UserSet::isDefaultUser( user ) ) user = defaultUser_;
    if( user.isEmpty() ) return;

    // emit command request
    emit commandRequested( AlienCommand( AlienCommand::JOB_LIST ) << user );

}

//________________________________________
void MainWindow::_installActions( void )
{

    Debug::Throw( "MainWindow::_installActions.\n" );

    // default user
    addAction( defaultUserAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Update Default User", this ) );
    connect( defaultUserAction_, SIGNAL( triggered() ), SLOT( requestDefaultUser( void ) ) );

    // user list
    addAction( userListAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Update User List", this ) );
    connect( userListAction_, SIGNAL( triggered() ), SLOT( requestUserList( void ) ) );

    // update
    addAction( updateAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Update", this ) );
    connect( updateAction_, SIGNAL( triggered( void ) ), SLOT( requestJobList( void ) ) );
    updateAction_->setShortcut( Qt::Key_F5 );

    // resubmit
    addAction( resubmitAction_ = new QAction( IconEngine::get( ICONS::RESUBMIT ), "Resubmit", this ) );
    resubmitAction_->setToolTip( "Re-submit failed selected jobs" );
    connect( resubmitAction_, SIGNAL( triggered( void ) ), SLOT( _resubmit( void ) ) );

    // trace
    addAction( traceAction_ = new QAction( IconEngine::get( ICONS::INFORMATION ), "Job Information", this ) );
    traceAction_->setToolTip( "Request information on current job" );
    connect( traceAction_, SIGNAL( triggered( void ) ), SLOT( _requestJobInformation( void ) ) );

    // kill
    addAction( killAction_ = new QAction( IconEngine::get( ICONS::KILL ), "Kill", this ) );
    killAction_->setToolTip( "Kill selected jobs" );
    connect( killAction_, SIGNAL( triggered( void ) ), SLOT( _kill( void ) ) );

    // clear
    addAction( clearAction_ = new QAction( IconEngine::get( ICONS::CLEAR ), "Clear", this ) );
    clearAction_->setToolTip( "Remove completed jobs from list" );
    clearAction_->setEnabled( false );
    connect( clearAction_, SIGNAL( triggered() ), SLOT( _clear() ) );

    // summary
    addAction( toggleSummaryAction_ = new QAction( "Summary", this ) );
    toggleSummaryAction_->setCheckable( true );
    toggleSummaryAction_->setChecked( true );
    connect( toggleSummaryAction_, SIGNAL( toggled( bool ) ), SLOT( _toggleSummary( bool ) ) );

    // add frame
    addAction( addFrameAction_ = new QAction( IconEngine::get( ICONS::ADD_TAB ), "Add Tab ...", this ) );
    addFrameAction_->setShortcut( Qt::CTRL+Qt::Key_T );
    connect( addFrameAction_, SIGNAL( triggered() ), SLOT( _addFrame( void ) ) );

    // remove frame
    addAction( removeFrameAction_ = new QAction( IconEngine::get( ICONS::REMOVE_TAB ), "Remove Tab", this ) );
    removeFrameAction_->setShortcut( Qt::CTRL+Qt::Key_W );
    connect( removeFrameAction_, SIGNAL( triggered() ), SLOT( _removeFrame( void ) ) );
    removeFrameAction_->setEnabled( false );

}


//________________________________________
void MainWindow::_setActiveFrame( JobManagerFrame& frame )
{
    Debug::Throw( "MainWindow::_setActiveFrame.\n" );
    if( activeFrame_ == &frame ) return;
    activeFrame_ = &frame;

    // update actions
    _updateActions();

}

//________________________________________
JobManagerFrame& MainWindow::_newFrame( void )
{

    Debug::Throw( "MainWindow::_newFrame.\n" );

    JobManagerFrame* frame( new JobManagerFrame() );

    // add actions to list
    frame->jobManager().list().menu().addAction( &frame->jobManager().list().findAction() );
    frame->jobManager().list().menu().addAction( &traceAction() );
    frame->jobManager().list().menu().addAction( &resubmitAction() );
    frame->jobManager().list().menu().addAction( &killAction() );
    frame->jobManager().list().menu().addAction( &updateAction() );
    frame->jobManager().list().menu().addSeparator();

    connect( frame->jobManager().list().selectionModel(), SIGNAL( selectionChanged ( const QItemSelection &, const QItemSelection & ) ), SLOT( _updateActions() ) );
    connect( &frame->jobManager(), SIGNAL( messageAvailable( const QString& ) ), &statusBar().label(), SLOT( setTextAndUpdate( const QString& ) ) );
    connect( &frame->jobManager().list(), SIGNAL( expanded( const QModelIndex& ) ), SLOT( _updateActions() ) );
    connect( &frame->jobManager().list(), SIGNAL( collapsed( const QModelIndex& ) ), SLOT( _updateActions() ) );
    connect( &frame->jobManager().list(), SIGNAL( activated( const QModelIndex& ) ), SLOT( _requestJobInformation() ) );

    // progressbar connections
    connect( &frame->jobManager(), SIGNAL( busy( int ) ), &statusBar(), SLOT( showProgressBar() ) );
    connect( &frame->jobManager(), SIGNAL( busy( int ) ), &statusBar().progressBar(), SLOT( setMaximum(int) ) );
    connect( &frame->jobManager(), SIGNAL( busy( int ) ), &statusBar().progressBar(), SLOT( reset() ) );

    connect( &frame->jobManager(), SIGNAL( progressAvailable( int ) ), &statusBar().progressBar(), SLOT( setValue(int) ) );
    connect( &frame->jobManager(), SIGNAL( idle( void ) ), &statusBar(), SLOT( showLabel() ) );

    // application connections
    Application& application( *static_cast<Application*>(Singleton::get().application()) );
    connect( &frame->jobManager(), SIGNAL( busy( int ) ), &application, SLOT( busy( void ) ) );
    connect( &frame->jobManager(), SIGNAL( idle( void ) ), &application, SLOT( idle( void ) ) );

    frame->jobManager().list().setFocus();

    BASE::Key::associate( this, frame );

    // update remove action
    if( BASE::KeySet<JobManagerFrame>( this ).size() > 1 )
    { removeFrameAction().setEnabled( true ); }

    return *frame;
}

//________________________________________
UserSet MainWindow::_users( void ) const
{

    UserSet out;

    // get list of frames
    BASE::KeySet<JobManagerFrame> frames( this );
    for( BASE::KeySet<JobManagerFrame>::const_iterator iter = frames.begin(); iter != frames.end(); ++iter )
    {
        QString user( (*iter)->jobManager().user() );
        if( UserSet::isDefaultUser( user ) && _hasDefaultUser() ) user = defaultUser_;
        out.insert( user );
    }

    return out;
}
